<?php
namespace Plugin\Controller;

use Plugin\Exception\Error;
use Plugin\Server\Response;
class Controller {

	private $route;

	public static function initialize($route = null){
		$controller = new self();
		$controller->setRoute($route);
		return $controller;
	}

	public function setRoute($route){
		$this->route = $route;
	}

	public function render(){
		$route= $this->route;
		list($app,$package,$control,$method) = explode(":",$route["controller"]);
		
		if (!$app || !$package || !$control || !$method) {
			Error::throw500("The render failed. Check your routes:\n". print_r($route,true));
		}

		$appController = $this->buildController($app,$package,$control);
		$appControllerName = get_class($appController);
		$methodName = "{$method}Action";

		if (!method_exists($appController, $methodName)){
			Error::throw500("Render Failed. Looks like the Method \"{$methodName}\" does not exist in {$appControllerName}");
		}
		
		//$response = call_user_method($methodName, $appController);// Don't forget the parameters if there are any
		$response = call_user_func_array(array($appController,$methodName), $route["params"]);
		
		if ( !($response instanceof Response)){
			Error::throw500("Looks like what you returned from {$appControllerName}::{$methodName} wasn't a Response.".'Use return $this->render(...)\n or return new Response(content)' );
		}

		return $response;
	}
	
	private function buildController($app,$package,$control){
		$classname = "src\\$app\\$package\\Controller\\{$control}Controller";
		if (!class_exists($classname)) {
			Error::throw500("The controller \"{$classname}\" does not exists. Check the spelling.");
		}
		return new $classname(); 
	}
	
	public function isHeadRequest(){
		return $_SERVER['REQUEST_METHOD'] == "HEAD";
	}
}