<?php
namespace Plugin\Routing;


use App\Config\Settings;
use Plugin\Exception\Error;
class Router {

	private static $rt;

	private $routes;
	private $uri;
	private $currentRoute;

	private static function initialize() {
		if (self::$rt){
			return ;
		}

		$rt = new self();
		
		$app = Settings::getAppsInfo();
		$routePath = $rt->getAppPaths($app);

		$allRoutes = array();
		foreach($routePath as $path){
			$filepath = ROOT_DIR."$path";
			if (file_exists($filepath)){
				include $filepath;
				$allRoutes = array_merge($allRoutes,$routes);
			}
		}
		if (empty($allRoutes)){
			Error::throw500("There are no routes available in your Application\n<br>Go to src/{Your App}/{Section}/Config/routes.php\n<br>if it does not exists, then create it.");
		}

		$rt->setUri($_SERVER["REQUEST_URI"]);
		$rt->setRoutes($allRoutes);

		self::$rt = $rt;
	}

	public static function getRoute(){
		self::initialize();
		$rt = self::$rt;
		$rt->processRoute();
		return $rt->getCurrentRoute();
	}

	public function setUri($uri){
		$this->uri = $uri;
	}

	public function setRoutes($routes){
		$this->routes = $routes;
	}

	private function getCurrentRoute(){
		return $this->currentRoute;
	}

	private function processRoute() {
		$urlParts = explode("/",$this->uri);
		$param = array();
		$currentRoute = null;
		$totalRoutes = count($this->routes);
		foreach($this->routes as $name => $value) {
		//for($index = 0;$index < $totalRoutes;$index++ ){
			$match = true;
			$patternSplit = explode("/",$this->routes[$name]["pattern"]);
			$l = count($patternSplit);
			if (count($urlParts) === count($patternSplit)){
				for($i=0;$i<$l;$i++){
					preg_match(	"/{(.*)}/", $patternSplit[$i],$matchtest);
					if (isset($matchtest[1])){
						$param[] = $urlParts[$i];
					}else {
						if ($patternSplit[$i] !== $urlParts[$i]){
							$match = false;
							break;
						}
					}
				}
			}else {
				$match = false;
			}

			if ($match === true){
				$currentRoute = $this->routes[$name];
				$currentRoute["params"] = $param;
				break;
			}

		}

		if (!$match) {
			Error::throw404("Page not found The current URL matches no routes");
		}

		$this->currentRoute = $currentRoute;
	}

	private function getAppPaths($apps){
		$paths = array();
		foreach($apps as $app => $sections) {
			foreach($sections as $name){
				$paths[] = "src/$app/$name/Config/routes.php";
			}
			
			
		}
		return $paths;
	}
}
